@@ -73,7 +73,9 @@ void draw_background(LoadGraph *graph) {
     guint i;
     unsigned num_bars;
     char *caption;
-    cairo_text_extents_t extents;
+    PangoLayout* layout;
+    PangoFontDescription* font_desc;
+    PangoRectangle extents;
 
     num_bars = graph->num_bars();
     graph->graph_dely = (graph->draw_height - 15) / num_bars; /* round to int to avoid AA blur */
@@ -93,6 +95,12 @@ void draw_background(LoadGraph *graph) {
     gdk_cairo_set_source_color (cr, &style->bg[GTK_STATE_NORMAL]);
     cairo_paint (cr);
 
+    layout = pango_cairo_create_layout (cr);
+    font_desc = pango_font_description_copy (style->font_desc);
+    pango_font_description_set_size (font_desc, 0.8 * graph->fontsize * PANGO_SCALE);
+    pango_layout_set_font_description (layout, font_desc);
+    pango_font_description_free (font_desc);
+
     /* draw frame */
     cairo_translate (cr, FRAME_WIDTH, FRAME_WIDTH);
 
@@ -104,7 +112,6 @@ void draw_background(LoadGraph *graph) {
 
     cairo_set_line_width (cr, 1.0);
     cairo_set_dash (cr, dash, 2, 0);
-    cairo_set_font_size (cr, graph->fontsize);
 
     for (i = 0; i <= num_bars; ++i) {
         double y;
@@ -121,15 +128,17 @@ void draw_background(LoadGraph *graph) {
             // operation orders matters so it's 0 if i == num_bars
             unsigned rate = graph->net.max - (i * graph->net.max / num_bars);
             const std::string caption(procman::format_network_rate(rate, graph->net.max));
-            cairo_text_extents (cr, caption.c_str(), &extents);
-            cairo_move_to (cr, graph->indent - extents.width + 20, y);
-            cairo_show_text (cr, caption.c_str());
+            pango_layout_set_text (layout, caption.c_str(), -1);
+            pango_layout_get_extents (layout, NULL, &extents);
+            cairo_move_to (cr, graph->indent - 1.0 * extents.width / PANGO_SCALE + 20, y - 1.0 * extents.height / PANGO_SCALE / 2);
+            pango_cairo_show_layout (cr, layout);
         } else {
             // operation orders matters so it's 0 if i == num_bars
-            caption = g_strdup_printf("%d %%", 100 - i * (100 / num_bars));
-            cairo_text_extents (cr, caption, &extents);
-            cairo_move_to (cr, graph->indent - extents.width + 20, y);
-            cairo_show_text (cr, caption);
+            caption = g_strdup_printf("%d%%", 100 - i * (100 / num_bars));
+            pango_layout_set_text (layout, caption, -1);
+            pango_layout_get_extents (layout, NULL, &extents);
+            cairo_move_to (cr, graph->indent - 1.0 * extents.width / PANGO_SCALE + 20, y - 1.0 * extents.height / PANGO_SCALE / 2);
+            pango_cairo_show_layout (cr, layout);
             g_free (caption);
         }
 
@@ -156,13 +165,14 @@ void draw_background(LoadGraph *graph) {
         else
             format = "%u";
         caption = g_strdup_printf(format, seconds);
-        cairo_text_extents (cr, caption, &extents);
-        cairo_move_to (cr, ((ceil(x) + 0.5) + graph->rmargin + graph->indent) - (extents.width/2), graph->draw_height);
+        pango_layout_set_text (layout, caption, -1);
+        pango_layout_get_extents (layout, NULL, &extents);
+        cairo_move_to (cr, ((ceil(x) + 0.5) + graph->rmargin + graph->indent) - (1.0 * extents.width / PANGO_SCALE/2), graph->draw_height - 1.0 * extents.height / PANGO_SCALE);
         gdk_cairo_set_source_color (cr, &style->fg[GTK_STATE_NORMAL]);
-        cairo_show_text (cr, caption);
+        pango_cairo_show_layout (cr, layout);
         g_free (caption);
     }
-
+    g_object_unref(layout);
     cairo_stroke (cr);
     cairo_destroy (cr);
 }
@@ -328,8 +338,12 @@ namespace
 
         used_text = procman::format_size(used);
         total_text = procman::format_size(total);
-        // xgettext: 540MiB (53 %) of 1.0 GiB
-        text = g_strdup_printf(_("%s (%.1f %%) of %s"), used_text, 100.0 * percent, total_text);
+        if (total == 0) {
+            text = g_strdup(_("not available"));
+        } else {
+            // xgettext: 540MiB (53 %) of 1.0 GiB
+            text = g_strdup_printf(_("%s (%.1f%%) of %s"), used_text, 100.0 * percent, total_text);
+        }
         gtk_label_set_text(label, text);
         g_free(used_text);
         g_free(total_text);
@@ -367,6 +381,33 @@ get_memory (LoadGraph *graph)
     graph->data[0][1] = swappercent;
 }
 
+/* Nice Numbers for Graph Labels after Paul Heckbert
+   nicenum: find a "nice" number approximately equal to x.
+   Round the number if round=1, take ceiling if round=0    */
+
+static double
+nicenum (double x, int round)
+{
+    int expv;				/* exponent of x */
+    double f;				/* fractional part of x */
+    double nf;				/* nice, rounded fraction */
+
+    expv = floor( log10(x) );
+    f = x/pow( 10.0, expv );		/* between 1 and 10 */
+    if (round) {
+        if ( f < 1.5 ) nf = 1.0;
+        else if ( f < 3.0 ) nf = 2.0;
+        else if ( f < 7.0 ) nf = 5.0;
+        else nf = 10.0;
+    } else {
+        if ( f <= 1.0 ) nf = 1.0;
+        else if ( f <= 2.0 ) nf = 2.0;
+        else if ( f <= 5.0 ) nf = 5.0;
+        else nf = 10.0;
+    }
+    return nf * pow(10.0, expv);
+}
+
 static void
 net_scale (LoadGraph *graph, guint64 din, guint64 dout)
 {
@@ -392,15 +433,20 @@ net_scale (LoadGraph *graph, guint64 din, guint64 dout)
     const guint64 bak_max(new_max);
 
     if (ProcData::get_instance()->config.network_in_bits) {
-        // TODO: fix logic to give a nice scale with bits
+        // nice number is for the ticks
+        unsigned ticks = graph->num_bars();
 
-        // round up to get some extra space
-        // yes, it can overflow
-        new_max = 1.1 * new_max;
-        // make sure max is not 0 to avoid / 0
-        // default to 125 bytes == 1kbit
-        new_max = std::max(new_max, G_GUINT64_CONSTANT(125));
+        // gets messy at low values due to division by 8
+        guint64 bit_max = std::max( new_max*8, G_GUINT64_CONSTANT(10000) );
 
+        // our tick size leads to max
+        double d = nicenum(bit_max/ticks, 0);
+        bit_max = ticks * d;
+        new_max = bit_max / 8;
+
+        procman_debug("bak*8 %" G_GUINT64_FORMAT ", ticks %d, d %f"
+                      ", bit_max %" G_GUINT64_FORMAT ", new_max %" G_GUINT64_FORMAT,
+                      bak_max*8, ticks, d, bit_max, new_max );
     } else {
         // round up to get some extra space
         // yes, it can overflow
@@ -742,8 +788,6 @@ load_graph_change_speed (LoadGraph *graph,
 
     graph->speed = new_speed;
 
-    g_assert(graph->timer_index);
-
     if(graph->timer_index) {
         g_source_remove (graph->timer_index);
         graph->timer_index = g_timeout_add (graph->speed / graph->frames_per_unit,
